@@ -1072,7 +1072,12 @@ editor_on_button_press (GtkWidget * signal_widget,
 					  "button_press_event");
 	  return handled;
 	}
+#ifdef USE_GNOME
+      else if (GTK_IS_FIXED (widget)
+	       || (GTK_IS_LAYOUT (widget) && !GNOME_IS_CANVAS (widget)))
+#else
       else if (GTK_IS_FIXED (widget) || GTK_IS_LAYOUT (widget))
+#endif
 	{
 	  add_widget_to_fixed (widget, x, y);
 	  gtk_signal_emit_stop_by_name (GTK_OBJECT (signal_widget),
@@ -1738,8 +1743,15 @@ editor_set_cursor (GtkWidget * signal_widget,
 #endif
   if (glade_palette_is_selector_on (GLADE_PALETTE (glade_palette)))
     {
+#ifdef USE_GNOME
       if (widget->parent
-	  && (GTK_IS_FIXED (widget->parent) || GTK_IS_LAYOUT (widget->parent)))
+	  && (GTK_IS_FIXED (widget)
+	      || (GTK_IS_LAYOUT (widget) && !GNOME_IS_CANVAS (widget))))
+#else
+      if (widget->parent
+	  && (GTK_IS_FIXED (widget->parent)
+	      || GTK_IS_LAYOUT (widget->parent)))
+#endif
 	{
 	  pos = get_position_in_widget (widget, x, y);
 	  switch (pos)
@@ -3084,6 +3096,12 @@ editor_can_delete_widget (GtkWidget * widget)
 {
   gchar *child_name;
 
+  /* Button & item labels are special - they can be deleted. */
+  if (GTK_IS_LABEL (widget)
+      && widget->parent && GB_IS_GB_WIDGET (widget->parent)
+      && (GTK_IS_BUTTON (widget->parent) || GTK_IS_ITEM (widget->parent)))
+    return NULL;
+
   /* Don't allow widgets which aren't GbWidgets to be deleted, since we know
      nothing about them. */
   if (!GB_IS_PLACEHOLDER (widget) && !GB_IS_GB_WIDGET (widget))
@@ -3105,12 +3123,15 @@ editor_can_delete_widget (GtkWidget * widget)
 	  || !strcmp (child_name, "Dialog:action_area")
 	  || !strcmp (child_name, "InputDialog:save_button")
 	  || !strcmp (child_name, "InputDialog:close_button")
+	  || !strcmp (child_name, "GtkCombo:entry")
 #ifdef USE_GNOME
 	  || !strcmp (child_name, "GnomeApp:dock")
 	  || !strcmp (child_name, "GnomeApp:appbar")
 	  || !strcmp (child_name, "GnomeDialog:vbox")
 	  || !strcmp (child_name, "GnomeDialog:action_area")
 	  || !strcmp (child_name, "GnomeDruidPageStandard:vbox")
+	  || !strcmp (child_name, "GnomeEntry:entry")
+	  || !strcmp (child_name, "GnomePixmapEntry:file-entry")
 #endif
 	  )
 	{
