@@ -19,7 +19,10 @@
 
 #include <string.h>
 
-#include <gtk/gtk.h>
+#include <gtk/gtkclist.h>
+#include <gtk/gtkhbox.h>
+#include <gtk/gtkmain.h>
+#include <gtk/gtkspinbutton.h>
 #include "../gb.h"
 
 /* Include the 21x21 icon pixmap for this widget, to be used in the palette */
@@ -37,25 +40,23 @@ static gchar *CListTitle = "CList:title";
 static gchar *Mode = "GtkCList::selection_mode";
 static gchar *Titles = "GtkCList::show_titles";
 static gchar *Shadow = "GtkCList::shadow_type";
-static gchar *HPolicy = "GtkCList::hscrollbar_policy";
-static gchar *VPolicy = "GtkCList::vscrollbar_policy";
 
 /* This is only used for loading & saving - it isn't displayed in the property
    editor. */
 static gchar *Cols = "GtkCList::columns";
 static gchar *ColumnWidths = "GtkCList::column_widths";
 
-static gchar *GbModeChoices[] =
+static const gchar *GbModeChoices[] =
 {"Single", "Browse", "Multiple",
  "Extended", NULL};
-static gint GbModeValues[] =
+static const gint GbModeValues[] =
 {
   GTK_SELECTION_SINGLE,
   GTK_SELECTION_BROWSE,
   GTK_SELECTION_MULTIPLE,
   GTK_SELECTION_EXTENDED
 };
-static gchar *GbModeSymbols[] =
+static const gchar *GbModeSymbols[] =
 {
   "GTK_SELECTION_SINGLE",
   "GTK_SELECTION_BROWSE",
@@ -63,10 +64,10 @@ static gchar *GbModeSymbols[] =
   "GTK_SELECTION_EXTENDED"
 };
 
-static gchar *GbShadowChoices[] =
+static const gchar *GbShadowChoices[] =
 {"None", "In", "Out",
  "Etched In", "Etched Out", NULL};
-static gint GbShadowValues[] =
+static const gint GbShadowValues[] =
 {
   GTK_SHADOW_NONE,
   GTK_SHADOW_IN,
@@ -74,7 +75,7 @@ static gint GbShadowValues[] =
   GTK_SHADOW_ETCHED_IN,
   GTK_SHADOW_ETCHED_OUT
 };
-static gchar *GbShadowSymbols[] =
+static const gchar *GbShadowSymbols[] =
 {
   "GTK_SHADOW_NONE",
   "GTK_SHADOW_IN",
@@ -83,29 +84,17 @@ static gchar *GbShadowSymbols[] =
   "GTK_SHADOW_ETCHED_OUT"
 };
 
-static gchar *GbPolicyChoices[] =
-{"Always", "Automatic", NULL};
-static gint GbPolicyValues[] =
-{
-  GTK_POLICY_ALWAYS,
-  GTK_POLICY_AUTOMATIC
-};
-static gchar *GbPolicySymbols[] =
-{
-  "GTK_POLICY_ALWAYS",
-  "GTK_POLICY_AUTOMATIC"
-};
-
 
 static void show_clist_dialog (GbWidgetNewData * data);
-static void on_clist_dialog_ok (GtkWidget * widget, GbWidgetNewData * data);
-static void on_clist_dialog_destroy (GtkWidget * widget, GbWidgetNewData * data);
+static void on_clist_dialog_ok (GtkWidget * widget,
+				GbWidgetNewData * data);
+static void on_clist_dialog_destroy (GtkWidget * widget,
+				     GbWidgetNewData * data);
 static GtkWidget *new_label (GtkWidget * parent);
+static GtkWidget *new_unnamed_label (GtkWidget * parent);
 
-static void hadjustment_value_changed (GtkAdjustment * adjustment,
-				       GtkWidget *clist);
-static void vadjustment_value_changed (GtkAdjustment * adjustment,
-				       GtkWidget *clist);
+static void gb_clist_adjustment_changed (GtkAdjustment *adjustment,
+					 GtkWidget     *widget);
 
 /******
  * NOTE: To use these functions you need to uncomment them AND add a pointer
@@ -123,8 +112,7 @@ GtkWidget *
 gb_clist_new (GbWidgetNewData * data)
 {
   GtkWidget *new_widget;
-  GtkAdjustment *adjustment;
-  gint cols = 0;
+  gint cols = 0, i;
 
   if (data->action == GB_LOADING)
     {
@@ -133,15 +121,14 @@ gb_clist_new (GbWidgetNewData * data)
 	cols = 1;
       new_widget = gtk_clist_new (cols);
 
-      /* Connect signals for redrawing. */
-      adjustment = gtk_range_get_adjustment (GTK_RANGE (GTK_CLIST (new_widget)->hscrollbar));
-      gtk_signal_connect_after (GTK_OBJECT (adjustment), "value_changed",
-				(GtkSignalFunc) hadjustment_value_changed,
-				new_widget);
-      adjustment = gtk_range_get_adjustment (GTK_RANGE (GTK_CLIST (new_widget)->vscrollbar));
-      gtk_signal_connect_after (GTK_OBJECT (adjustment), "value_changed",
-				(GtkSignalFunc) vadjustment_value_changed,
-				new_widget);
+      /* GtkCList has problems if the title buttons aren't created. */
+      for (i = 0; i < cols; i++)
+	{
+	  gtk_clist_set_column_widget (GTK_CLIST (new_widget), i,
+                                       new_unnamed_label (new_widget));
+	  gtk_clist_set_column_width (GTK_CLIST (new_widget), i, 80);
+          editor_add_mouse_signals_to_existing (GTK_CLIST (new_widget)->column[i].button);
+	}
 
       return new_widget;
     }
@@ -156,29 +143,17 @@ gb_clist_new (GbWidgetNewData * data)
 static void
 show_clist_dialog (GbWidgetNewData * data)
 {
-  GtkWidget *window, *vbox, *hbox, *separator, *hbbox, *label, *spinbutton;
-  GtkWidget *button;
+  GtkWidget *dialog, *vbox, *hbox, *label, *spinbutton;
   GtkObject *adjustment;
 
-  window = gtk_window_new (GTK_WINDOW_DIALOG);
-  gtk_window_position (GTK_WINDOW (window), GTK_WIN_POS_MOUSE);
-  gtk_window_set_title (GTK_WINDOW (window), _("New columned list"));
-
-  gtk_signal_connect_object (GTK_OBJECT (window), "delete_event",
-			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			     GTK_OBJECT (window));
-  gtk_signal_connect (GTK_OBJECT (window), "destroy",
-		      GTK_SIGNAL_FUNC (on_clist_dialog_destroy),
-		      data);
-
-  vbox = gtk_vbox_new (FALSE, 5);
-  gtk_container_add (GTK_CONTAINER (window), vbox);
-  /*gtk_container_border_width (GTK_CONTAINER (vbox), 10); */
-  gtk_widget_show (vbox);
+  dialog = glade_util_create_dialog (_("New columned list"), data->parent,
+				     on_clist_dialog_ok, data, &vbox);
+  gtk_signal_connect (GTK_OBJECT (dialog), "destroy",
+		      GTK_SIGNAL_FUNC (on_clist_dialog_destroy), data);
 
   hbox = gtk_hbox_new (FALSE, 5);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 5);
-  gtk_container_border_width (GTK_CONTAINER (hbox), 10);
+  gtk_container_set_border_width (GTK_CONTAINER (hbox), 10);
   gtk_widget_show (hbox);
 
   label = gtk_label_new (_("Number of columns:"));
@@ -188,56 +163,30 @@ show_clist_dialog (GbWidgetNewData * data)
   adjustment = gtk_adjustment_new (3, 1, 100, 1, 10, 10);
   spinbutton = gtk_spin_button_new (GTK_ADJUSTMENT (adjustment), 1, 0);
   /* save pointer to spinbutton so we can find it easily in the OK handler */
-  gtk_object_set_data (GTK_OBJECT (window), "spinbutton", spinbutton);
+  gtk_object_set_data (GTK_OBJECT (dialog), "cols", spinbutton);
   gtk_box_pack_start (GTK_BOX (hbox), spinbutton, TRUE, TRUE, 5);
   gtk_widget_set_usize (spinbutton, 50, -1);
   gtk_widget_grab_focus (spinbutton);
   gtk_widget_show (spinbutton);
 
-  separator = gtk_hseparator_new ();
-  gtk_box_pack_start (GTK_BOX (vbox), separator, TRUE, TRUE, 5);
-  gtk_widget_show (separator);
-
-  hbbox = gtk_hbutton_box_new ();
-  gtk_box_pack_start (GTK_BOX (vbox), hbbox, TRUE, TRUE, 5);
-  gtk_container_border_width (GTK_CONTAINER (vbox), 10);
-  gtk_widget_show (hbbox);
-
-  button = gtk_button_new_with_label (_("OK"));
-  gtk_container_add (GTK_CONTAINER (hbbox), button);
-  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-  gtk_widget_grab_default (button);
-  gtk_widget_show (button);
-  gtk_signal_connect (GTK_OBJECT (button), "clicked",
-		      GTK_SIGNAL_FUNC (on_clist_dialog_ok),
-		      data);
-
-  button = gtk_button_new_with_label (_("Cancel"));
-  gtk_container_add (GTK_CONTAINER (hbbox), button);
-  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-  gtk_widget_show (button);
-  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
-			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			     GTK_OBJECT (window));
-
-  gtk_widget_show (window);
-  gtk_grab_add (window);
+  gtk_widget_show (dialog);
+  gtk_grab_add (dialog);
 }
 
 
 static void
-on_clist_dialog_ok (GtkWidget * widget, GbWidgetNewData * data)
+on_clist_dialog_ok (GtkWidget * widget,
+		    GbWidgetNewData * data)
 {
-  GtkWidget *new_widget, *spinbutton, *window;
-  GtkAdjustment *adjustment;
+  GtkWidget *new_widget, *spinbutton, *dialog, *label;
   gint cols, i;
 
-  window = gtk_widget_get_toplevel (widget);
+  dialog = gtk_widget_get_toplevel (widget);
 
   /* Only call callback if placeholder/fixed widget is still there */
   if (gb_widget_can_finish_new (data))
     {
-      spinbutton = gtk_object_get_data (GTK_OBJECT (window), "spinbutton");
+      spinbutton = gtk_object_get_data (GTK_OBJECT (dialog), "cols");
       g_return_if_fail (spinbutton != NULL);
       cols = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (spinbutton));
 
@@ -245,30 +194,22 @@ on_clist_dialog_ok (GtkWidget * widget, GbWidgetNewData * data)
       gtk_clist_column_titles_show (GTK_CLIST (new_widget));
       for (i = 0; i < cols; i++)
 	{
+	  label = new_label (new_widget);
 	  gtk_clist_set_column_widget (GTK_CLIST (new_widget), i,
-				       new_label (new_widget));
+				       label);
 	  gtk_clist_set_column_width (GTK_CLIST (new_widget), i, 80);
 	}
 
-      /* Connect signals for redrawing. */
-      adjustment = gtk_range_get_adjustment (GTK_RANGE (GTK_CLIST (new_widget)->hscrollbar));
-      gtk_signal_connect_after (GTK_OBJECT (adjustment), "value_changed",
-				(GtkSignalFunc) hadjustment_value_changed,
-				new_widget);
-      adjustment = gtk_range_get_adjustment (GTK_RANGE (GTK_CLIST (new_widget)->vscrollbar));
-      gtk_signal_connect_after (GTK_OBJECT (adjustment), "value_changed",
-				(GtkSignalFunc) vadjustment_value_changed,
-				new_widget);
-
       gb_widget_initialize (new_widget, data);
       (*data->callback) (new_widget, data);
     }
-  gtk_widget_destroy (window);
+  gtk_widget_destroy (dialog);
 }
 
 
 static void
-on_clist_dialog_destroy (GtkWidget * widget, GbWidgetNewData * data)
+on_clist_dialog_destroy (GtkWidget * widget,
+			 GbWidgetNewData * data)
 {
   gb_widget_free_new_data (data);
   gtk_grab_remove (widget);
@@ -282,57 +223,37 @@ new_label (GtkWidget * parent)
 
   label = gb_widget_new ("GtkLabel", parent);
   g_return_val_if_fail (label != NULL, NULL);
-  gtk_object_set_data (GTK_OBJECT (label), GB_CHILD_NAME_KEY,
-		       g_strdup (CListTitle));
+  gb_widget_set_child_name (label, CListTitle);
   return label;
 }
 
 
-/* These two functions redraw the clist when the scrollbars are used, so
-   that the selection rectangle doesn't get messed up - the clist uses
-   gdk_window_copy_area() which will copy the selection handles which we
-   do not want to happen. */
-static void
-hadjustment_value_changed (GtkAdjustment * adjustment,
-			   GtkWidget *clist)
+GtkWidget *
+new_unnamed_label (GtkWidget * parent)
 {
-  gint x, y, w, h;
+  GtkWidget *label;
 
-  x = clist->allocation.x;
-  y = clist->allocation.y;
-  w = clist->allocation.width;
-  h = clist->allocation.height;
-
-  editor_refresh_widget_area (clist, x, y, w, GB_CORNER_HEIGHT);
-  editor_refresh_widget_area (clist, x, y + h - GB_CORNER_HEIGHT,
-			      w, GB_CORNER_HEIGHT);
-  gtk_widget_draw (clist, NULL);
-  /* FIXME: GTK bug workaround to draw CList children. */
-  gtk_widget_draw_children (clist);
+  label = gb_widget_new_full ("GtkLabel", FALSE, parent, NULL, 0, 0, NULL,
+			      GB_CREATING, NULL);
+  g_return_val_if_fail (label != NULL, NULL);
+  gb_widget_set_child_name (label, CListTitle);
+  return label;
 }
 
 
+/* This just redraws the entire widget when it is scrolled, to make sure that
+   it isn't messed up due to our selection handles. */
 static void
-vadjustment_value_changed (GtkAdjustment * adjustment,
-			   GtkWidget *clist)
+gb_clist_adjustment_changed (GtkAdjustment *adjustment,
+			     GtkWidget     *widget)
 {
-  gint x, y, w, h;
-
-  x = clist->allocation.x;
-  y = clist->allocation.y;
-  w = clist->allocation.width;
-  h = clist->allocation.height;
-
-  editor_refresh_widget_area (clist, x, y, GB_CORNER_WIDTH, h);
-  editor_refresh_widget_area (clist, x + w - GB_CORNER_WIDTH, y,
-			      GB_CORNER_WIDTH, h);
-  gtk_widget_draw (clist, NULL);
-  /* FIXME: GTK bug workaround to draw CList children. */
-  gtk_widget_draw_children (clist);
+  /* We check that this is a widget in the interface being created rather
+     than part of Glade's interface. */
+  if (GB_IS_GB_WIDGET (widget))
+    gtk_widget_queue_clear (widget);
 }
 
 
-
 /*
  * Creates the components needed to edit the extra properties of this widget.
  */
@@ -347,12 +268,6 @@ gb_clist_create_properties (GtkWidget * widget, GbWidgetCreateArgData * data)
   property_add_choice (Shadow, _("Shadow:"),
 		       _("The type of shadow of the columned list's border"),
 		       GbShadowChoices);
-  property_add_choice (HPolicy, _("H Policy:"),
-		       _("When the horizontal scrollbar will be shown"),
-		       GbPolicyChoices);
-  property_add_choice (VPolicy, _("V Policy:"),
-		       _("When the vertical scrollbar will be shown"),
-		       GbPolicyChoices);
 }
 
 
@@ -407,17 +322,6 @@ gb_clist_get_properties (GtkWidget * widget, GbWidgetGetArgData * data)
       if (GbShadowValues[i] == GTK_CLIST (widget)->shadow_type)
 	gb_widget_output_choice (data, Shadow, i, GbShadowSymbols[i]);
     }
-
-  for (i = 0; i < sizeof (GbPolicyValues) / sizeof (GbPolicyValues[0]); i++)
-    {
-      if (GbPolicyValues[i] == GTK_CLIST (widget)->hscrollbar_policy)
-	gb_widget_output_choice (data, HPolicy, i, GbPolicySymbols[i]);
-    }
-  for (i = 0; i < sizeof (GbPolicyValues) / sizeof (GbPolicyValues[0]); i++)
-    {
-      if (GbPolicyValues[i] == GTK_CLIST (widget)->vscrollbar_policy)
-	gb_widget_output_choice (data, VPolicy, i, GbPolicySymbols[i]);
-    }
 }
 
 
@@ -430,11 +334,9 @@ static void
 gb_clist_set_properties (GtkWidget * widget, GbWidgetSetArgData * data)
 {
   gchar *widths, *pos, *mode;
-  gchar *shadow, *hpolicy, *vpolicy;
-  gboolean myApply, titles;
+  gchar *shadow;
+  gboolean titles;
   gint col, w, i;
-  GtkPolicyType hpolicy_value = GTK_POLICY_AUTOMATIC;
-  GtkPolicyType vpolicy_value = GTK_POLICY_AUTOMATIC;
 
   if (data->action == GB_LOADING)
     {
@@ -486,29 +388,12 @@ gb_clist_set_properties (GtkWidget * widget, GbWidgetSetArgData * data)
 	  if (!strcmp (shadow, GbShadowChoices[i])
 	      || !strcmp (shadow, GbShadowSymbols[i]))
 	    {
-	      gtk_clist_set_border (GTK_CLIST (widget), GbShadowValues[i]);
+	      gtk_clist_set_shadow_type (GTK_CLIST (widget),
+					 GbShadowValues[i]);
 	      break;
 	    }
 	}
     }
-
-  hpolicy = gb_widget_input_choice (data, HPolicy);
-  myApply = data->apply;
-  vpolicy = gb_widget_input_choice (data, VPolicy);
-  if (data->apply || myApply)
-    {
-      for (i = 0; i < sizeof (GbPolicyValues) / sizeof (GbPolicyValues[0]); i
-	   ++)
-	{
-	  if (!strcmp (hpolicy, GbPolicyChoices[i])
-	      || !strcmp (hpolicy, GbPolicySymbols[i]))
-	    hpolicy_value = GbPolicyValues[i];
-	  if (!strcmp (vpolicy, GbPolicyChoices[i])
-	      || !strcmp (vpolicy, GbPolicySymbols[i]))
-	    vpolicy_value = GbPolicyValues[i];
-	}
-      gtk_clist_set_policy (GTK_CLIST (widget), vpolicy_value, hpolicy_value);
-    }
 }
 
 
@@ -527,6 +412,35 @@ gb_clist_set_properties (GtkWidget * widget, GbWidgetSetArgData * data)
  */
 
 
+void
+gb_clist_add_child (GtkWidget *widget, GtkWidget *child, GbWidgetSetArgData *data)
+{
+  gchar *child_name = load_get_value (data, "child_name");
+
+  if (child_name && (!strcmp (child_name, CListTitle)))
+    {
+      /* We store the last column title read in 'last_child' */
+      gint col = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (widget),
+                                                       "last_child"));
+
+      gtk_clist_set_column_widget (GTK_CLIST (widget), col, child);
+      gtk_object_set_data (GTK_OBJECT (widget), "last_child",
+                           GINT_TO_POINTER (col + 1));
+
+      /* We need to add signals to the clist button, just in case the
+         title widget has no window and so doesn't get signals itself.
+         Since Clist always creates 1 button initially, the signals would
+         be added to this button in gb_widget_new, so we could skip it,
+         but it doesn't hurt. */
+      editor_add_mouse_signals_to_existing (GTK_CLIST (widget)->column[col].button);
+    }
+  else
+    {
+      g_warning ("Adding unknown child to a GtkCList");
+      gtk_container_add (GTK_CONTAINER (widget), child);
+    }
+}
+
 
 /*
  * Writes the source code needed to create this widget.
@@ -536,7 +450,7 @@ gb_clist_set_properties (GtkWidget * widget, GbWidgetSetArgData * data)
 static void
 gb_clist_write_source (GtkWidget * widget, GbWidgetWriteSourceData * data)
 {
-  gint col, i, hpolicy = 0, vpolicy = 0;
+  gint col, i;
 
   if (data->create_widget)
     {
@@ -546,7 +460,8 @@ gb_clist_write_source (GtkWidget * widget, GbWidgetWriteSourceData * data)
 
   /* We reset the last_child index, so as the title widgets are written out
      they will start at column 0. */
-  gtk_object_set_data (GTK_OBJECT (widget), "last_child", GINT_TO_POINTER (-1));
+  gtk_object_set_data (GTK_OBJECT (widget), "last_child",
+		       GINT_TO_POINTER (-1));
 
   gb_widget_write_standard_source (widget, data);
 
@@ -581,31 +496,89 @@ gb_clist_write_source (GtkWidget * widget, GbWidgetWriteSourceData * data)
 	   ++)
 	{
 	  if (GbShadowValues[i] == GTK_CLIST (widget)->shadow_type)
-	    source_add (data, "  gtk_clist_set_border (GTK_CLIST (%s), %s);\n",
+	    source_add (data,
+			"  gtk_clist_set_shadow_type (GTK_CLIST (%s), %s);\n",
 			data->wname, GbShadowSymbols[i]);
 	}
     }
+}
 
-  if (GTK_CLIST (widget)->hscrollbar_policy != GTK_POLICY_ALWAYS
-      || GTK_CLIST (widget)->vscrollbar_policy != GTK_POLICY_ALWAYS)
+
+/* Outputs source to add a child widget to a CList. */
+static void
+gb_clist_write_add_child_source (GtkWidget * parent,
+				 const gchar *parent_name,
+				 GtkWidget *child,
+				 GbWidgetWriteSourceData * data)
+{
+  gchar *child_name;
+
+  child_name = gb_widget_get_child_name (child);
+
+  /* See if this is a title widget. */
+  if (child_name && (!strcmp (child_name, CListTitle)))
     {
-      for (i = 0; i < sizeof (GbPolicyValues) / sizeof (GbPolicyValues[0]); i
-	   ++)
-	{
-	  if (GbPolicyValues[i] == GTK_CLIST (widget)->hscrollbar_policy)
-	    hpolicy = i;
-	  if (GbPolicyValues[i] == GTK_CLIST (widget)->vscrollbar_policy)
-	    vpolicy = i;
-	}
+      /* We store the last column title written in 'last_child' */
+      gint col = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (parent),
+						       "last_child"));
+      source_add (data,
+		  "  gtk_clist_set_column_widget (GTK_CLIST (%s), %i, %s);\n",
+		  parent_name, col + 1, data->wname);
 
-      source_add (data, "  gtk_clist_set_policy (GTK_CLIST (%s), %s, %s);\n",
-		  data->wname, GbPolicySymbols[vpolicy],
-		  GbPolicySymbols[hpolicy]);
+      gtk_object_set_data (GTK_OBJECT (parent), "last_child",
+			   GINT_TO_POINTER (col + 1));
+    }
+  else
+    {
+      g_warning ("Unknown CList child widgetL %s", data->wname);
+      source_add (data, "  gtk_container_add (GTK_CONTAINER (%s), %s);\n",
+		  parent_name, data->wname);
     }
 }
 
 
 
+static gboolean
+gb_clist_emission_hook (GtkObject *object,
+			guint signal_id,
+			guint n_params,
+			GtkArg *params,
+			gpointer data)
+{
+  GtkObject *hadjustment, *vadjustment;
+  GtkObject *old_hadjustment, *old_vadjustment;
+
+  g_return_val_if_fail (GTK_IS_CLIST (object), FALSE);
+
+  hadjustment = GTK_VALUE_OBJECT (params[0]);
+  vadjustment = GTK_VALUE_OBJECT (params[1]);
+
+  old_hadjustment = gtk_object_get_data (object, "scrollhadjustment");
+  if (hadjustment != old_hadjustment)
+    {
+      gtk_object_set_data (object, "scrollhadjustment", hadjustment);
+
+      if (hadjustment)
+	gtk_signal_connect (hadjustment, "value_changed",
+			    (GtkSignalFunc) gb_clist_adjustment_changed,
+			    object);
+    }
+
+  old_vadjustment = gtk_object_get_data (object, "scrollvadjustment");
+  if (vadjustment != old_vadjustment)
+    {
+      gtk_object_set_data (object, "scrollvadjustment", vadjustment);
+
+      if (vadjustment)
+	gtk_signal_connect (vadjustment, "value_changed",
+			    (GtkSignalFunc) gb_clist_adjustment_changed,
+			    object);
+    }
+
+  return TRUE;
+}
+
+
 /*
  * Initializes the GbWidget structure.
  * I've placed this at the end of the file so we don't have to include
@@ -614,9 +587,18 @@ gb_clist_write_source (GtkWidget * widget, GbWidgetWriteSourceData * data)
 GbWidget *
 gb_clist_init ()
 {
+  GtkWidgetClass *klass;
+
   /* Initialise the GTK type */
   gtk_clist_get_type ();
 
+  /* Add a signal emission hook so we can connect signal handlers to the
+     scrollbar adjustments to redraw the clist when necessary. This will also
+     work for subclasses of GtkCList. */
+  klass = gtk_type_class (gtk_clist_get_type ());
+  gtk_signal_add_emission_hook (klass->set_scroll_adjustments_signal,
+				gb_clist_emission_hook, NULL);
+
   /* Initialize the GbWidget structure */
   gb_widget_init_struct (&gbwidget);
 
@@ -626,10 +608,12 @@ gb_clist_init ()
 
   /* Fill in any functions that this GbWidget has */
   gbwidget.gb_widget_new = gb_clist_new;
+  gbwidget.gb_widget_add_child = gb_clist_add_child;
   gbwidget.gb_widget_create_properties = gb_clist_create_properties;
   gbwidget.gb_widget_get_properties = gb_clist_get_properties;
   gbwidget.gb_widget_set_properties = gb_clist_set_properties;
   gbwidget.gb_widget_write_source = gb_clist_write_source;
+  gbwidget.gb_widget_write_add_child_source = gb_clist_write_add_child_source;
 /*
    gbwidget.gb_widget_create_popup_menu = gb_clist_create_popup_menu;
  */
