@@ -46,10 +46,7 @@ static gchar *Arrows = "GtkCombo::use_arrows";
 static gchar *Always = "GtkCombo::use_arrows_always";
 static gchar *Items = "GtkCombo::items";
 
-/* Maximum length of text containing all combo popdown strings. */
-#define	ITEMS_BUFFER_SIZE	8192
-
-static void add_label (GtkWidget * widget, gchar * items);
+static void add_label (GtkWidget * widget, GString * items);
 
 static gint is_simple_combo (GtkWidget * widget);
 static void is_simple_combo_callback (GtkWidget * widget,
@@ -117,8 +114,6 @@ gb_combo_create_properties (GtkWidget * widget, GbWidgetCreateArgData * data)
 static void
 gb_combo_get_properties (GtkWidget * widget, GbWidgetGetArgData * data)
 {
-  gchar items[ITEMS_BUFFER_SIZE];
-
   gb_widget_output_bool (data, ValueInList,
 			 gtk_object_get_data (GTK_OBJECT (widget), ValueInList)
 			 != NULL ? TRUE : FALSE);
@@ -131,44 +126,26 @@ gb_combo_get_properties (GtkWidget * widget, GbWidgetGetArgData * data)
 
   if (is_simple_combo (widget) >= 0)
     {
-      /* Clear the buffer and make sure it's seen as valid. */
-      items[0] = '\0';
+      GString *items;
+
+      items = g_string_new ("");
       gtk_container_foreach (GTK_CONTAINER (GTK_COMBO (widget)->list),
-			     (GtkCallback) add_label, &items);
-      if (items[0] == -1)
-	{
-	  gb_widget_output_text (data, Items, "");
-	  if (data->action == GB_SHOWING)
-	    property_set_sensitive (Items, FALSE);
-	}
-      else
-	{
-	  gb_widget_output_translatable_text (data, Items, items);
-	  if (data->action == GB_SHOWING)
-	    property_set_sensitive (Items, TRUE);
-	}
+			     (GtkCallback) add_label, items);
+      gb_widget_output_translatable_text (data, Items, items->str);
+      g_string_free (items, TRUE);
     }
 }
 
 /* Adds combo label text to buffer, or sets first char to -1 if it won't fit.
  */
 static void
-add_label (GtkWidget * widget, gchar * items)
+add_label (GtkWidget * widget, GString *items)
 {
   gchar *label_text;
 
-  /* Return if list has already been marked invalid (i.e. it is too big). */
-  if (items[0] == -1)
-    return;
-
   gtk_label_get (GTK_LABEL (GTK_BIN (widget)->child), &label_text);
-  if (strlen (items) + strlen (label_text) + 2 < ITEMS_BUFFER_SIZE)
-    {
-      strcat (items, label_text);
-      strcat (items, "\n");
-    }
-  else
-    items[0] = -1;
+  g_string_append (items, label_text);
+  g_string_append_c (items, '\n');
 }
 
 
@@ -189,8 +166,11 @@ is_simple_combo (GtkWidget * widget)
 static void
 is_simple_combo_callback (GtkWidget * widget, gint * num_children)
 {
+  /* If we've already found an item which isn't a simple label, just return. */
   if (*num_children == -1)
     return;
+
+  /* If the items isn't a simple label, set num_children to -1, else add 1. */
   if (!GTK_IS_LABEL (GTK_BIN (widget)->child))
     *num_children = -1;
   else
