@@ -377,7 +377,8 @@ void
 gb_box_set_child_properties (GtkWidget *widget, GtkWidget *child,
 			     GbWidgetSetArgData *data)
 {
-  gint position, padding, old_padding;
+  gint position, padding;
+  guint old_padding;
   gboolean expand, fill, pack, set_child_packing = FALSE;
   gboolean old_expand, old_fill;
   GtkPackType old_pack_type;
