@@ -19,7 +19,11 @@
 
 #include <math.h>
 
-#include <gtk/gtk.h>
+#include <gtk/gtkimage.h>
+#include <gtk/gtklabel.h>
+#include <gtk/gtkmain.h>
+#include <gtk/gtkradiobutton.h>
+#include <gtk/gtkspinbutton.h>
 #include "../gb.h"
 
 /* Include the 21x21 icon pixmap for this widget, to be used in the palette */
@@ -50,8 +54,10 @@ static gchar *VisualSystem = "GDK_VISUAL_SYSTEM";
 static gchar *VisualBest = "GDK_VISUAL_BEST";
 
 static void show_image_dialog (GbWidgetNewData * data);
-static void on_image_dialog_ok (GtkWidget * widget, GbWidgetNewData * data);
-static void on_image_dialog_destroy (GtkWidget * widget, GbWidgetNewData * data);
+static void on_image_dialog_ok (GtkWidget * widget,
+				GbWidgetNewData * data);
+static void on_image_dialog_destroy (GtkWidget * widget,
+				     GbWidgetNewData * data);
 
 /******
  * NOTE: To use these functions you need to uncomment them AND add a pointer
@@ -150,31 +156,18 @@ gb_image_new (GbWidgetNewData * data)
 static void
 show_image_dialog (GbWidgetNewData * data)
 {
-  GtkWidget *window, *vbox, *separator, *hbbox, *label, *spinbutton;
-  GtkWidget *button, *table;
+  GtkWidget *dialog, *vbox, *table, *label, *spinbutton, *button;
   GtkObject *adjustment;
   GSList *group;
 
-  window = gtk_window_new (GTK_WINDOW_DIALOG);
-  gtk_window_position (GTK_WINDOW (window), GTK_WIN_POS_MOUSE);
-  gtk_window_set_title (GTK_WINDOW (window), _("New image"));
-
-  gtk_signal_connect_object (GTK_OBJECT (window), "delete_event",
-			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			     GTK_OBJECT (window));
-  gtk_signal_connect (GTK_OBJECT (window), "destroy",
-		      GTK_SIGNAL_FUNC (on_image_dialog_destroy),
-		      data);
-
-  vbox = gtk_vbox_new (FALSE, 5);
-  gtk_container_add (GTK_CONTAINER (window), vbox);
-  /*gtk_container_border_width (GTK_CONTAINER (vbox), 10); */
-  gtk_widget_show (vbox);
+  dialog = glade_util_create_dialog (_("New image"), data->parent,
+				     on_image_dialog_ok, data, &vbox);
+  gtk_signal_connect (GTK_OBJECT (dialog), "destroy",
+		      GTK_SIGNAL_FUNC (on_image_dialog_destroy), data);
 
   /* Image width & height spinbuttons */
   table = gtk_table_new (4, 7, FALSE);
   gtk_box_pack_start (GTK_BOX (vbox), table, TRUE, TRUE, 5);
-  /*gtk_container_border_width (GTK_CONTAINER (table), 4); */
   gtk_widget_show (table);
 
   label = gtk_label_new (_("Image Width:"));
@@ -186,7 +179,7 @@ show_image_dialog (GbWidgetNewData * data)
   adjustment = gtk_adjustment_new (100, 1, 10000, 1, 5, 10);
   spinbutton = gtk_spin_button_new (GTK_ADJUSTMENT (adjustment), 1, 0);
   /* save pointer to entry so we can find it easily in the OK handler */
-  gtk_object_set_data (GTK_OBJECT (window), "spinbutton", spinbutton);
+  gtk_object_set_data (GTK_OBJECT (dialog), "width", spinbutton);
   gtk_table_attach (GTK_TABLE (table), spinbutton, 1, 7, 0, 1,
 		    GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 5, 5);
   gtk_widget_grab_focus (spinbutton);
@@ -202,7 +195,7 @@ show_image_dialog (GbWidgetNewData * data)
   adjustment = gtk_adjustment_new (100, 1, 10000, 1, 5, 10);
   spinbutton = gtk_spin_button_new (GTK_ADJUSTMENT (adjustment), 1, 0);
   /* save pointer to entry so we can find it easily in the OK handler */
-  gtk_object_set_data (GTK_OBJECT (window), "spinbutton2", spinbutton);
+  gtk_object_set_data (GTK_OBJECT (dialog), "height", spinbutton);
   gtk_table_attach (GTK_TABLE (table), spinbutton, 1, 7, 1, 2,
 		    GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 5, 5);
   gtk_widget_show (spinbutton);
@@ -218,21 +211,21 @@ show_image_dialog (GbWidgetNewData * data)
   gtk_table_attach (GTK_TABLE (table), button, 1, 3, 2, 3,
 		    GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 5, 5);
   gtk_widget_show (button);
-  gtk_object_set_data (GTK_OBJECT (window), "button1", button);
+  gtk_object_set_data (GTK_OBJECT (dialog), "normal", button);
   group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
 
   button = gtk_radio_button_new_with_label (group, _("Shared"));
   gtk_table_attach (GTK_TABLE (table), button, 3, 5, 2, 3,
 		    GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 5, 5);
   gtk_widget_show (button);
-  gtk_object_set_data (GTK_OBJECT (window), "button2", button);
+  gtk_object_set_data (GTK_OBJECT (dialog), "shared", button);
   group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
 
   button = gtk_radio_button_new_with_label (group, _("Fastest"));
   gtk_table_attach (GTK_TABLE (table), button, 5, 7, 2, 3,
 		    GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 5, 5);
   gtk_widget_show (button);
-  gtk_object_set_data (GTK_OBJECT (window), "button3", button);
+  gtk_object_set_data (GTK_OBJECT (dialog), "fastest", button);
 
   /* Visual - System/Best */
   label = gtk_label_new (_("Visual:"));
@@ -245,44 +238,17 @@ show_image_dialog (GbWidgetNewData * data)
   gtk_table_attach (GTK_TABLE (table), button, 1, 4, 3, 4,
 		    GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 5, 5);
   gtk_widget_show (button);
-  gtk_object_set_data (GTK_OBJECT (window), "button4", button);
+  gtk_object_set_data (GTK_OBJECT (dialog), "system", button);
   group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
 
   button = gtk_radio_button_new_with_label (group, _("Best"));
   gtk_table_attach (GTK_TABLE (table), button, 4, 7, 3, 4,
 		    GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 5, 5);
   gtk_widget_show (button);
-  gtk_object_set_data (GTK_OBJECT (window), "button5", button);
+  gtk_object_set_data (GTK_OBJECT (dialog), "best", button);
 
-  /* Separator & buttons */
-  separator = gtk_hseparator_new ();
-  gtk_box_pack_start (GTK_BOX (vbox), separator, TRUE, TRUE, 5);
-  gtk_widget_show (separator);
-
-  hbbox = gtk_hbutton_box_new ();
-  gtk_box_pack_start (GTK_BOX (vbox), hbbox, TRUE, TRUE, 5);
-  gtk_container_border_width (GTK_CONTAINER (vbox), 10);
-  gtk_widget_show (hbbox);
-
-  button = gtk_button_new_with_label (_("OK"));
-  gtk_container_add (GTK_CONTAINER (hbbox), button);
-  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-  gtk_widget_grab_default (button);
-  gtk_widget_show (button);
-  gtk_signal_connect (GTK_OBJECT (button), "clicked",
-		      GTK_SIGNAL_FUNC (on_image_dialog_ok),
-		      data);
-
-  button = gtk_button_new_with_label (_("Cancel"));
-  gtk_container_add (GTK_CONTAINER (hbbox), button);
-  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-  gtk_widget_show (button);
-  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
-			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			     GTK_OBJECT (window));
-
-  gtk_widget_show (window);
-  gtk_grab_add (window);
+  gtk_widget_show (dialog);
+  gtk_grab_add (dialog);
 }
 
 
@@ -302,16 +268,16 @@ on_image_dialog_ok (GtkWidget * widget, GbWidgetNewData * data)
   if (gb_widget_can_finish_new (data))
     {
       /* Get image width & height */
-      spinbutton = gtk_object_get_data (GTK_OBJECT (window), "spinbutton");
+      spinbutton = gtk_object_get_data (GTK_OBJECT (window), "width");
       g_return_if_fail (spinbutton != NULL);
       width = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (spinbutton));
 
-      spinbutton = gtk_object_get_data (GTK_OBJECT (window), "spinbutton2");
+      spinbutton = gtk_object_get_data (GTK_OBJECT (window), "height");
       g_return_if_fail (spinbutton != NULL);
       height = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (spinbutton));
 
       /* Get image type */
-      button = gtk_object_get_data (GTK_OBJECT (window), "button1");
+      button = gtk_object_get_data (GTK_OBJECT (window), "normal");
       g_return_if_fail (button != NULL);
       if (GTK_TOGGLE_BUTTON (button)->active)
 	{
@@ -319,7 +285,7 @@ on_image_dialog_ok (GtkWidget * widget, GbWidgetNewData * data)
 	  type_symbol = ImageNormal;
 	}
 
-      button = gtk_object_get_data (GTK_OBJECT (window), "button2");
+      button = gtk_object_get_data (GTK_OBJECT (window), "shared");
       g_return_if_fail (button != NULL);
       if (GTK_TOGGLE_BUTTON (button)->active)
 	{
@@ -327,7 +293,7 @@ on_image_dialog_ok (GtkWidget * widget, GbWidgetNewData * data)
 	  type_symbol = ImageShared;
 	}
 
-      button = gtk_object_get_data (GTK_OBJECT (window), "button3");
+      button = gtk_object_get_data (GTK_OBJECT (window), "fastest");
       g_return_if_fail (button != NULL);
       if (GTK_TOGGLE_BUTTON (button)->active)
 	{
@@ -336,7 +302,7 @@ on_image_dialog_ok (GtkWidget * widget, GbWidgetNewData * data)
 	}
 
       /* Get image visual */
-      button = gtk_object_get_data (GTK_OBJECT (window), "button4");
+      button = gtk_object_get_data (GTK_OBJECT (window), "system");
       g_return_if_fail (button != NULL);
       if (GTK_TOGGLE_BUTTON (button)->active)
 	{
@@ -345,7 +311,7 @@ on_image_dialog_ok (GtkWidget * widget, GbWidgetNewData * data)
 	  visual_symbol = VisualSystem;
 	}
 
-      button = gtk_object_get_data (GTK_OBJECT (window), "button5");
+      button = gtk_object_get_data (GTK_OBJECT (window), "best");
       g_return_if_fail (button != NULL);
       if (GTK_TOGGLE_BUTTON (button)->active)
 	{
@@ -373,7 +339,8 @@ on_image_dialog_ok (GtkWidget * widget, GbWidgetNewData * data)
 
 
 static void
-on_image_dialog_destroy (GtkWidget * widget, GbWidgetNewData * data)
+on_image_dialog_destroy (GtkWidget * widget,
+			 GbWidgetNewData * data)
 {
   gb_widget_free_new_data (data);
   gtk_grab_remove (widget);
@@ -388,10 +355,10 @@ gb_image_create_properties (GtkWidget * widget, GbWidgetCreateArgData * data)
 {
   property_add_float_range (XAlign, _("X Align:"),
 			    _("The horizontal alignment"),
-			    0, 1, 0.01, 1, 0, 2);
+			    0, 1, 0.01, 0.1, 0.01, 2);
   property_add_float_range (YAlign, _("Y Align:"),
 			    _("The vertical alignment"),
-			    0, 1, 0.01, 1, 0, 2);
+			    0, 1, 0.01, 0.1, 0.01, 2);
   property_add_int_range (XPad, _("X Pad:"), _("The horizontal padding"),
 			  0, 1000, 1, 10, 1);
   property_add_int_range (YPad, _("Y Pad:"), _("The vertical padding"),
@@ -419,9 +386,11 @@ gb_image_get_properties (GtkWidget * widget, GbWidgetGetArgData * data)
       gb_widget_output_int (data, Height, GPOINTER_TO_INT (gtk_object_get_data
 					    (GTK_OBJECT (widget), Height)));
       gb_widget_output_string (data, Type,
-			   gtk_object_get_data (GTK_OBJECT (widget), Type));
+			       gtk_object_get_data (GTK_OBJECT (widget),
+						    Type));
       gb_widget_output_string (data, Visual,
-			 gtk_object_get_data (GTK_OBJECT (widget), Visual));
+			       gtk_object_get_data (GTK_OBJECT (widget),
+						    Visual));
     }
 }
 
@@ -436,18 +405,36 @@ gb_image_set_properties (GtkWidget * widget, GbWidgetSetArgData * data)
 {
   gfloat xalign, yalign;
   gint xpad, ypad;
-  gboolean myApply;
+  gboolean set_alignment = FALSE, set_padding = FALSE;
 
   xalign = gb_widget_input_float (data, XAlign);
-  myApply = data->apply;	/* save Apply flag */
+  if (data->apply)
+    set_alignment = TRUE;
+  else
+    xalign = GTK_MISC (widget)->xalign;
+
   yalign = gb_widget_input_float (data, YAlign);
-  if (data->apply || myApply)
+  if (data->apply)
+    set_alignment = TRUE;
+  else
+    yalign = GTK_MISC (widget)->yalign;
+
+  if (set_alignment)
     gtk_misc_set_alignment (GTK_MISC (widget), xalign, yalign);
 
   xpad = gb_widget_input_int (data, XPad);
-  myApply = data->apply;	/* save Apply flag */
+  if (data->apply)
+    set_padding = TRUE;
+  else
+    xpad = GTK_MISC (widget)->xpad;
+
   ypad = gb_widget_input_int (data, YPad);
-  if (data->apply || myApply)
+  if (data->apply)
+    set_padding = TRUE;
+  else
+    ypad = GTK_MISC (widget)->ypad;
+
+  if (set_padding)
     gtk_misc_set_padding (GTK_MISC (widget), xpad, ypad);
 }
 
