@@ -17,7 +17,15 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include <gtk/gtk.h>
+#include <string.h>
+
+#include <gtk/gtkhbox.h>
+#include <gtk/gtklabel.h>
+#include <gtk/gtkmain.h>
+#include <gtk/gtkmenu.h>
+#include <gtk/gtkmenuitem.h>
+#include <gtk/gtkspinbutton.h>
+#include <gtk/gtktable.h>
 #include "../gb.h"
 
 /* Include the 21x21 icon pixmap for this widget, to be used in the palette */
@@ -29,16 +37,37 @@
  */
 static GbWidget gbwidget;
 
-static gchar *Rows = "GtkTable::rows";
-static gchar *Columns = "GtkTable::columns";
-static gchar *Homogeneous = "GtkTable::homogeneous";
-static gchar *RowSpacing = "GtkTable::row_spacing";
-static gchar *ColSpacing = "GtkTable::column_spacing";
+static const gchar *Rows = "GtkTable::rows";
+static const gchar *Columns = "GtkTable::columns";
+static const gchar *Homogeneous = "GtkTable::homogeneous";
+static const gchar *RowSpacing = "GtkTable::row_spacing";
+static const gchar *ColSpacing = "GtkTable::column_spacing";
 
+/* For children of a table */
+static const gchar *GbCellX = "GtkTableChild::cell_x";
+static const gchar *GbCellY = "GtkTableChild::cell_y";
+static const gchar *GbColSpan = "GtkTableChild::col_span";
+static const gchar *GbRowSpan = "GtkTableChild::row_span";
+static const gchar *GbXPad = "GtkTableChild::xpad";
+static const gchar *GbYPad = "GtkTableChild::ypad";
+static const gchar *GbXExpand = "GtkTableChild::xexpand";
+static const gchar *GbYExpand = "GtkTableChild::yexpand";
+static const gchar *GbXShrink = "GtkTableChild::xshrink";
+static const gchar *GbYShrink = "GtkTableChild::yshrink";
+static const gchar *GbXFill = "GtkTableChild::xfill";
+static const gchar *GbYFill = "GtkTableChild::yfill";
+
+
+/* These are used to return what is in a table cell */
+#define GB_CELL_EMPTY		1
+#define GB_CELL_WIDGET		2
+#define GB_CELL_PLACEHOLDER	3
 
 static void show_table_dialog (GbWidgetNewData * data);
-static void on_table_dialog_ok (GtkWidget * widget, GbWidgetNewData * data);
-static void on_table_dialog_destroy (GtkWidget * widget, GbWidgetNewData * data);
+static void on_table_dialog_ok (GtkWidget * widget,
+				GbWidgetNewData * data);
+static void on_table_dialog_destroy (GtkWidget * widget,
+				     GbWidgetNewData * data);
 
 static void update_table_size (GtkWidget * table, gint rows, gint cols);
 
@@ -54,6 +83,15 @@ static void gb_table_insert_row_or_col (GtkWidget * table, gint row, gint col);
 static void gb_table_delete_row (GtkWidget * menuitem, GtkWidget * widget);
 static void gb_table_delete_column (GtkWidget * menuitem, GtkWidget * widget);
 
+static gint gb_table_is_cell_occupied (GtkWidget * table, gint row, gint col);
+static void gb_table_remove_placeholders (GtkWidget * table,
+					  gint row, gint col);
+static void gb_table_split_placeholder (GtkWidget * table,
+					GtkWidget * placeholder,
+					gint left, gint right,
+					gint top, gint bottom,
+					gint skip_row, gint skip_col);
+
 /******
  * NOTE: To use these functions you need to uncomment them AND add a pointer
  * to the function in the GbWidget struct at the end of this file.
@@ -90,30 +128,18 @@ gb_table_new (GbWidgetNewData * data)
 static void
 show_table_dialog (GbWidgetNewData * data)
 {
-  GtkWidget *window, *vbox, *separator, *hbbox, *label, *spinbutton;
-  GtkWidget *button, *table;
+  GtkWidget *dialog, *vbox, *table, *label, *spinbutton;
   GtkObject *adjustment;
 
-  window = gtk_window_new (GTK_WINDOW_DIALOG);
-  gtk_window_position (GTK_WINDOW (window), GTK_WIN_POS_MOUSE);
-  gtk_window_set_title (GTK_WINDOW (window), _("New table"));
-
-  gtk_signal_connect_object (GTK_OBJECT (window), "delete_event",
-			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			     GTK_OBJECT (window));
-  gtk_signal_connect (GTK_OBJECT (window), "destroy",
-		      GTK_SIGNAL_FUNC (on_table_dialog_destroy),
-		      data);
-
-  vbox = gtk_vbox_new (FALSE, 5);
-  gtk_container_add (GTK_CONTAINER (window), vbox);
-  /*gtk_container_border_width (GTK_CONTAINER (vbox), 10); */
-  gtk_widget_show (vbox);
+  dialog = glade_util_create_dialog (_("New table"), data->parent,
+				     on_table_dialog_ok, data, &vbox);
+  gtk_signal_connect (GTK_OBJECT (dialog), "destroy",
+		      GTK_SIGNAL_FUNC (on_table_dialog_destroy), data);
 
   /* Rows label & entry */
   table = gtk_table_new (2, 2, FALSE);
   gtk_box_pack_start (GTK_BOX (vbox), table, TRUE, TRUE, 5);
-  gtk_container_border_width (GTK_CONTAINER (table), 4);
+  gtk_container_set_border_width (GTK_CONTAINER (table), 4);
   gtk_widget_show (table);
 
   label = gtk_label_new (_("Number of rows:"));
@@ -125,7 +151,7 @@ show_table_dialog (GbWidgetNewData * data)
   adjustment = gtk_adjustment_new (3, 1, 100, 1, 10, 10);
   spinbutton = gtk_spin_button_new (GTK_ADJUSTMENT (adjustment), 1, 0);
   /* save pointer to entry so we can find it easily in the OK handler */
-  gtk_object_set_data (GTK_OBJECT (window), "spinbutton", spinbutton);
+  gtk_object_set_data (GTK_OBJECT (dialog), "rows", spinbutton);
   gtk_table_attach (GTK_TABLE (table), spinbutton, 1, 2, 0, 1,
 		    GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 4, 4);
   gtk_widget_set_usize (spinbutton, 50, -1);
@@ -142,40 +168,14 @@ show_table_dialog (GbWidgetNewData * data)
   adjustment = gtk_adjustment_new (3, 1, 100, 1, 10, 10);
   spinbutton = gtk_spin_button_new (GTK_ADJUSTMENT (adjustment), 1, 0);
   /* save pointer to entry so we can find it easily in the OK handler */
-  gtk_object_set_data (GTK_OBJECT (window), "spinbutton2", spinbutton);
+  gtk_object_set_data (GTK_OBJECT (dialog), "cols", spinbutton);
   gtk_table_attach (GTK_TABLE (table), spinbutton, 1, 2, 1, 2,
 		    GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 4, 4);
   gtk_widget_set_usize (spinbutton, 50, -1);
   gtk_widget_show (spinbutton);
 
-  separator = gtk_hseparator_new ();
-  gtk_box_pack_start (GTK_BOX (vbox), separator, TRUE, TRUE, 5);
-  gtk_widget_show (separator);
-
-  hbbox = gtk_hbutton_box_new ();
-  gtk_box_pack_start (GTK_BOX (vbox), hbbox, TRUE, TRUE, 5);
-  gtk_container_border_width (GTK_CONTAINER (vbox), 10);
-  gtk_widget_show (hbbox);
-
-  button = gtk_button_new_with_label (_("OK"));
-  gtk_container_add (GTK_CONTAINER (hbbox), button);
-  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-  gtk_widget_grab_default (button);
-  gtk_widget_show (button);
-  gtk_signal_connect (GTK_OBJECT (button), "clicked",
-		      GTK_SIGNAL_FUNC (on_table_dialog_ok),
-		      data);
-
-  button = gtk_button_new_with_label (_("Cancel"));
-  gtk_container_add (GTK_CONTAINER (hbbox), button);
-  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-  gtk_widget_show (button);
-  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
-			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			     GTK_OBJECT (window));
-
-  gtk_widget_show (window);
-  gtk_grab_add (window);
+  gtk_widget_show (dialog);
+  gtk_grab_add (dialog);
 }
 
 
@@ -190,11 +190,11 @@ on_table_dialog_ok (GtkWidget * widget, GbWidgetNewData * data)
   /* Only call callback if placeholder/fixed widget is still there */
   if (gb_widget_can_finish_new (data))
     {
-      spinbutton = gtk_object_get_data (GTK_OBJECT (window), "spinbutton");
+      spinbutton = gtk_object_get_data (GTK_OBJECT (window), "rows");
       g_return_if_fail (spinbutton != NULL);
       rows = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (spinbutton));
 
-      spinbutton = gtk_object_get_data (GTK_OBJECT (window), "spinbutton2");
+      spinbutton = gtk_object_get_data (GTK_OBJECT (window), "cols");
       g_return_if_fail (spinbutton != NULL);
       cols = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (spinbutton));
 
@@ -203,9 +203,11 @@ on_table_dialog_ok (GtkWidget * widget, GbWidgetNewData * data)
 	{
 	  for (col = 0; col < cols; col++)
 	    {
-	      gtk_table_attach_defaults (GTK_TABLE (new_widget),
-					 editor_new_placeholder (),
-					 col, col + 1, row, row + 1);
+	      gtk_table_attach (GTK_TABLE (new_widget),
+				editor_new_placeholder (),
+				col, col + 1, row, row + 1,
+				GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL,
+				0, 0);
 	    }
 	}
       gb_widget_initialize (new_widget, data);
@@ -216,14 +218,14 @@ on_table_dialog_ok (GtkWidget * widget, GbWidgetNewData * data)
 
 
 static void
-on_table_dialog_destroy (GtkWidget * widget, GbWidgetNewData * data)
+on_table_dialog_destroy (GtkWidget * widget,
+			 GbWidgetNewData * data)
 {
   gb_widget_free_new_data (data);
   gtk_grab_remove (widget);
 }
 
 
-
 /*
  * Creates the components needed to edit the extra properties of this widget.
  */
@@ -236,7 +238,7 @@ gb_table_create_properties (GtkWidget * widget, GbWidgetCreateArgData * data)
   property_add_int_range (Columns, _("Columns:"),
 			  _("The number of columns in the table"),
 			  1, 1000, 1, 10, 1);
-  property_add_bool (Homogeneous, _("Homogeneous"),
+  property_add_bool (Homogeneous, _("Homogeneous:"),
 		     _("If the children should all be the same size"));
   property_add_int_range (RowSpacing, _("Row Spacing:"),
 			  _("The space between each row"),
@@ -271,23 +273,27 @@ gb_table_get_properties (GtkWidget * widget, GbWidgetGetArgData * data)
 static void
 gb_table_set_properties (GtkWidget * widget, GbWidgetSetArgData * data)
 {
-  gboolean homogeneous, myApply;
+  gboolean homogeneous, update_size = FALSE;
   gint rows, cols, row_spacing, column_spacing;
 
   rows = gb_widget_input_int (data, Rows);
-  myApply = data->apply;
+  if (data->apply)
+    update_size = TRUE;
+  else
+    rows = GTK_TABLE (widget)->nrows;
+
   cols = gb_widget_input_int (data, Columns);
-  if (myApply || data->apply)
-    {
-      update_table_size (widget, rows, cols);
-    }
+  if (data->apply)
+    update_size = TRUE;
+  else
+    cols = GTK_TABLE (widget)->ncols;
+
+  if (update_size)
+    update_table_size (widget, rows, cols);
 
   homogeneous = gb_widget_input_bool (data, Homogeneous);
   if (data->apply)
-    {
-      GTK_TABLE (widget)->homogeneous = homogeneous;
-      gtk_widget_queue_resize (widget);
-    }
+    gtk_table_set_homogeneous (GTK_TABLE (widget), homogeneous);
 
   row_spacing = gb_widget_input_int (data, RowSpacing);
   if (data->apply)
@@ -316,8 +322,6 @@ update_table_size (GtkWidget * table, gint rows, gint cols)
   if (GTK_TABLE (table)->nrows == rows && GTK_TABLE (table)->ncols == cols)
     return;
 
-  gtk_container_block_resize (GTK_CONTAINER (table));
-
   children = GTK_TABLE (table)->children;
   while (children)
     {
@@ -340,19 +344,220 @@ update_table_size (GtkWidget * table, gint rows, gint cols)
     }
 
   /* This is in ../gbwidget.c. It will expand the dimensions if necessary. */
-  gb_widget_update_table_placeholders (table, rows, cols);
+  gb_table_update_placeholders (table, rows, cols);
 
   /* This bit is especially naughty. It makes sure the table shrinks,
      so we don't get extra spacings on the end. */
   GTK_TABLE (table)->nrows = rows;
   GTK_TABLE (table)->ncols = cols;
 
-  gtk_container_unblock_resize (GTK_CONTAINER (table));
   /* We clear the selection since it isn't displayed properly. */
   editor_clear_selection (NULL);
 }
 
 
+/*
+ * Creates the child packing properties for children of this widget.
+ */
+static void
+gb_table_create_child_properties (GtkWidget * widget,
+				  GbWidgetCreateChildArgData * data)
+{
+  property_add_int_range (GbCellX, _("Cell X:"),
+			  _("The left edge of the widget in the table"),
+			  0, 10000, 1, 10, 1);
+  property_add_int_range (GbCellY, _("Cell Y:"),
+			  _("The top edge of the widget in the table"),
+			  0, 10000, 1, 10, 1);
+
+  property_add_int_range (GbColSpan, _("Col Span:"),
+		 _("The number of columns spanned by the widget in the table"),
+			  1, 10000, 1, 10, 1);
+  property_add_int_range (GbRowSpan, _("Row Span:"),
+		    _("The number of rows spanned by the widget in the table"),
+			  1, 10000, 1, 10, 1);
+  property_add_int_range (GbXPad, _("H Padding:"),
+			  _("The horizontal padding"),
+			  0, 10000, 1, 10, 1);
+  property_add_int_range (GbYPad, _("V Padding:"),
+			  _("The vertical padding"),
+			  0, 10000, 1, 10, 1);
+  property_add_bool (GbXExpand, _("X Expand:"),
+		     _("Set True to let the widget expand horizontally"));
+  property_add_bool (GbYExpand, _("Y Expand:"),
+		     _("Set True to let the widget expand vertically"));
+  property_add_bool (GbXShrink, _("X Shrink:"),
+		     _("Set True to let the widget shrink horizontally"));
+  property_add_bool (GbYShrink, _("Y Shrink:"),
+		     _("Set True to let the widget shrink vertically"));
+  property_add_bool (GbXFill, _("X Fill:"),
+	   _("Set True to let the widget fill its horizontal allocated area"));
+  property_add_bool (GbYFill, _("Y Fill:"),
+	     _("Set True to let the widget fill its vertical allocated area"));
+}
+
+
+static void
+gb_table_get_child_properties (GtkWidget *widget, GtkWidget *child,
+			       GbWidgetGetArgData *data)
+{
+  GtkTableChild *table_child;
+
+  table_child = glade_util_find_table_child (GTK_TABLE (widget), child);
+  g_return_if_fail (table_child != NULL);
+
+  if (data->action == GB_SAVING)
+    save_start_tag (data, "child");
+
+  if (data->action == GB_SAVING)
+    {
+      /* We use left/right/top/bottom_attach here as they are used in GTK */
+      gb_widget_output_int (data, "GtkWidget::left_attach",
+			    table_child->left_attach);
+      gb_widget_output_int (data, "GtkWidget::right_attach",
+			    table_child->right_attach);
+      gb_widget_output_int (data, "GtkWidget::top_attach",
+			    table_child->top_attach);
+      gb_widget_output_int (data, "GtkWidget::bottom_attach",
+			    table_child->bottom_attach);
+    }
+  else
+    {
+      gb_widget_output_int (data, GbCellX, table_child->left_attach);
+      gb_widget_output_int (data, GbCellY, table_child->top_attach);
+      gb_widget_output_int (data, GbColSpan,
+			    table_child->right_attach
+			    - table_child->left_attach);
+      gb_widget_output_int (data, GbRowSpan,
+			    table_child->bottom_attach
+			    - table_child->top_attach);
+    }
+  gb_widget_output_int (data, GbXPad, table_child->xpadding);
+  gb_widget_output_int (data, GbYPad, table_child->ypadding);
+  gb_widget_output_bool (data, GbXExpand, table_child->xexpand);
+  gb_widget_output_bool (data, GbYExpand, table_child->yexpand);
+  gb_widget_output_bool (data, GbXShrink, table_child->xshrink);
+  gb_widget_output_bool (data, GbYShrink, table_child->yshrink);
+  gb_widget_output_bool (data, GbXFill, table_child->xfill);
+  gb_widget_output_bool (data, GbYFill, table_child->yfill);
+
+  if (data->action == GB_SAVING)
+    save_end_tag (data, "child");
+}
+
+
+static void
+gb_table_set_child_properties (GtkWidget *widget, GtkWidget *child,
+			       GbWidgetSetArgData *data)
+{
+  GtkTableChild *tchild;
+  gint xpad, ypad, left_attach, right_attach, top_attach, bottom_attach;
+  gint xexpand, yexpand, xshrink, yshrink, xfill, yfill;
+
+  tchild = glade_util_find_table_child (GTK_TABLE (widget), child);
+  g_return_if_fail (tchild != NULL);
+
+  xpad = gb_widget_input_int (data, GbXPad);
+  if (!data->apply)
+    xpad = tchild->xpadding;
+
+  ypad = gb_widget_input_int (data, GbYPad);
+  if (!data->apply)
+    ypad = tchild->ypadding;
+
+  xexpand = (gb_widget_input_bool (data, GbXExpand)) ? GTK_EXPAND : 0;
+  if (!data->apply)
+    xexpand = tchild->xexpand ? GTK_EXPAND : 0;
+
+  yexpand = (gb_widget_input_bool (data, GbYExpand)) ? GTK_EXPAND : 0;
+  if (!data->apply)
+    yexpand = tchild->yexpand ? GTK_EXPAND : 0;
+
+  xshrink = (gb_widget_input_bool (data, GbXShrink)) ? GTK_SHRINK : 0;
+  if (!data->apply)
+    xshrink = tchild->xshrink ? GTK_SHRINK : 0;
+
+  yshrink = (gb_widget_input_bool (data, GbYShrink)) ? GTK_SHRINK : 0;
+  if (!data->apply)
+    yshrink = tchild->yshrink ? GTK_SHRINK : 0;
+
+  xfill = (gb_widget_input_bool (data, GbXFill)) ? GTK_FILL : 0;
+  if (!data->apply)
+    xfill = tchild->xfill ? GTK_FILL : 0;
+
+  yfill = (gb_widget_input_bool (data, GbYFill)) ? GTK_FILL : 0;
+  if (!data->apply)
+    yfill = tchild->yfill ? GTK_FILL : 0;
+
+  if (data->action == GB_APPLYING)
+    {
+      left_attach = gb_widget_input_int (data, GbCellX);
+      if (!data->apply)
+	left_attach = tchild->left_attach;
+
+      top_attach = gb_widget_input_int (data, GbCellY);
+      if (!data->apply)
+	top_attach = tchild->top_attach;
+
+      right_attach = gb_widget_input_int (data, GbColSpan) + left_attach;
+      if (!data->apply)
+	right_attach = left_attach + (tchild->right_attach
+				      - tchild->left_attach);
+
+      bottom_attach = gb_widget_input_int (data, GbRowSpan) + top_attach;
+      if (!data->apply)
+	bottom_attach = top_attach + (tchild->bottom_attach
+				      - tchild->top_attach);
+    }
+  else
+    {
+      left_attach = gb_widget_input_int (data, "GtkWidget::left_attach");
+      if (!data->apply)
+	left_attach = tchild->left_attach;
+
+      top_attach = gb_widget_input_int (data, "GtkWidget::top_attach");
+      if (!data->apply)
+	top_attach = tchild->top_attach;
+
+      right_attach = gb_widget_input_int (data, "GtkWidget::right_attach");
+      if (!data->apply)
+	right_attach = tchild->right_attach;
+
+      bottom_attach = gb_widget_input_int (data, "GtkWidget::bottom_attach");
+      if (!data->apply)
+	bottom_attach = tchild->bottom_attach;
+    }
+
+  if (right_attach <= left_attach)
+    right_attach = left_attach + 1;
+  if (bottom_attach <= top_attach)
+    bottom_attach = top_attach + 1;
+
+  if (xpad != tchild->xpadding || ypad != tchild->ypadding
+      || (xexpand && !tchild->xexpand) || (!xexpand && tchild->xexpand)
+      || (yexpand && !tchild->yexpand) || (!yexpand && tchild->yexpand)
+      || (xshrink && !tchild->xshrink) || (!xshrink && tchild->xshrink)
+      || (yshrink && !tchild->yshrink) || (!yshrink && tchild->yshrink)
+      || (xfill   && !tchild->xfill)   || (!xfill   && tchild->xfill)
+      || (yfill   && !tchild->yfill)   || (!yfill   && tchild->yfill)
+      || left_attach   != tchild->left_attach
+      || right_attach  != tchild->right_attach
+      || top_attach    != tchild->top_attach
+      || bottom_attach != tchild->bottom_attach)
+    {
+      gtk_widget_ref (child);
+      gtk_container_remove (GTK_CONTAINER (widget), child);
+      gtk_table_attach (GTK_TABLE (widget), child,
+			left_attach, right_attach, top_attach, bottom_attach,
+			xexpand | xshrink | xfill, yexpand | yshrink | yfill,
+			xpad, ypad);
+      gtk_widget_unref (child);
+      if (data->action == GB_APPLYING)
+	gb_table_update_placeholders (widget, -1, -1);
+    }
+}
+
+
 /*
  * Adds menu items to a context menu which is just about to appear!
  * Add commands to aid in editing a GtkTable, with signals pointing to
@@ -363,7 +568,11 @@ gb_table_create_popup_menu (GtkWidget * widget, GbWidgetCreateMenuData * data)
 {
   GtkWidget *menuitem;
 
-  g_return_if_fail (data->child != NULL);
+  /* It is possible that the mouse button was clicked outside a child
+     (e.g. beneath all the children), in which case we return. We could
+     still support adding rows/cols. */
+  if (data->child == NULL)
+    return;
 
   menuitem = gtk_menu_item_new_with_label (_("Insert Row Before"));
   gtk_widget_show (menuitem);
@@ -415,7 +624,7 @@ gb_table_insert_row_before (GtkWidget * menuitem, GtkWidget * widget)
 
   table = widget->parent;
   g_return_if_fail (GTK_IS_TABLE (table));
-  tchild = gb_table_find_child (GTK_TABLE (table), widget);
+  tchild = glade_util_find_table_child (GTK_TABLE (table), widget);
   g_return_if_fail (tchild != NULL);
   gb_table_insert_row_or_col (table, tchild->top_attach, -1);
 }
@@ -429,7 +638,7 @@ gb_table_insert_row_after (GtkWidget * menuitem, GtkWidget * widget)
 
   table = widget->parent;
   g_return_if_fail (GTK_IS_TABLE (table));
-  tchild = gb_table_find_child (GTK_TABLE (table), widget);
+  tchild = glade_util_find_table_child (GTK_TABLE (table), widget);
   g_return_if_fail (tchild != NULL);
   gb_table_insert_row_or_col (table, tchild->bottom_attach, -1);
 }
@@ -443,7 +652,7 @@ gb_table_insert_column_before (GtkWidget * menuitem, GtkWidget * widget)
 
   table = widget->parent;
   g_return_if_fail (GTK_IS_TABLE (table));
-  tchild = gb_table_find_child (GTK_TABLE (table), widget);
+  tchild = glade_util_find_table_child (GTK_TABLE (table), widget);
   g_return_if_fail (tchild != NULL);
   gb_table_insert_row_or_col (table, -1, tchild->left_attach);
 }
@@ -457,7 +666,7 @@ gb_table_insert_column_after (GtkWidget * menuitem, GtkWidget * widget)
 
   table = widget->parent;
   g_return_if_fail (GTK_IS_TABLE (table));
-  tchild = gb_table_find_child (GTK_TABLE (table), widget);
+  tchild = glade_util_find_table_child (GTK_TABLE (table), widget);
   g_return_if_fail (tchild != NULL);
   gb_table_insert_row_or_col (table, -1, tchild->right_attach);
 }
@@ -473,8 +682,6 @@ gb_table_insert_row_or_col (GtkWidget * table, gint row, gint col)
   GtkWidget *widget, *tmp_label;
   gint rows, cols;
 
-  gtk_container_block_resize (GTK_CONTAINER (table));
-
   /* This relies on knowing the internals of GtkTable, so it is fast.
      First it adds a simple label at the bottom right of the new table size,
      to ensure that the table grows. Then it removes the label, and moves
@@ -515,8 +722,7 @@ gb_table_insert_row_or_col (GtkWidget * table, gint row, gint col)
   gtk_container_remove (GTK_CONTAINER (table), tmp_label);
 
   /* This fills any empty cells with placeholders. */
-  gb_widget_update_table_placeholders (table, -1, -1);
-  gtk_container_unblock_resize (GTK_CONTAINER (table));
+  gb_table_update_placeholders (table, -1, -1);
 
   /* If the tables properties are currently shown, update rows/cols. */
   if (property_get_widget () == table)
@@ -544,7 +750,7 @@ gb_table_delete_row (GtkWidget * menuitem, GtkWidget * widget)
   if (nrows == 0)
     return;
   ncols = GTK_TABLE (table)->ncols;
-  tchild = gb_table_find_child (GTK_TABLE (table), widget);
+  tchild = glade_util_find_table_child (GTK_TABLE (table), widget);
   g_return_if_fail (tchild != NULL);
   row = tchild->top_attach;
 
@@ -591,7 +797,7 @@ gb_table_delete_column (GtkWidget * menuitem, GtkWidget * widget)
   ncols = GTK_TABLE (table)->ncols - 1;
   if (ncols == 0)
     return;
-  tchild = gb_table_find_child (GTK_TABLE (table), widget);
+  tchild = glade_util_find_table_child (GTK_TABLE (table), widget);
   g_return_if_fail (tchild != NULL);
   col = tchild->left_attach;
 
@@ -625,6 +831,185 @@ gb_table_delete_column (GtkWidget * menuitem, GtkWidget * widget)
 }
 
 
+/* This ensures that placeholders are placed in every unoccupied cell.
+   If rows and cols are not -1, then they are the new dimensions of the table.
+   FIXME: This is very inefficient.
+ */
+void
+gb_table_update_placeholders (GtkWidget * table, gint rows, gint cols)
+{
+  gint row, col, cell_contents;
+  guchar *rows_occupied, *cols_occupied;
+  GList *children;
+  GtkTableChild *child;
+
+  if (rows == -1)
+    rows = GTK_TABLE (table)->nrows;
+  if (cols == -1)
+    cols = GTK_TABLE (table)->ncols;
+
+  /* These hold flags to indicate which rows/cols have widgets in them. */
+  rows_occupied = g_new0 (guchar, rows);
+  cols_occupied = g_new0 (guchar, cols);
+
+  for (row = 0; row < rows; row++)
+    {
+      for (col = 0; col < cols; col++)
+	{
+	  /* Find out what is in the cell */
+	  cell_contents = gb_table_is_cell_occupied (table, row, col);
+	  if (cell_contents == GB_CELL_WIDGET)
+	    {
+	      /* If cell is occupied, delete any placeholders there. If a
+		 placeholder occupies the cell but spans multiple rows/cols
+		 split it into single cells but without a placeholder in this
+		 cell */
+	      gb_table_remove_placeholders (table, row, col);
+	      rows_occupied[row] = 1;
+	      cols_occupied[col] = 1;
+	    }
+	  else if (cell_contents == GB_CELL_EMPTY)
+	    {
+	      /* If the cell is empty, put a placeholder in it */
+	      gtk_table_attach (GTK_TABLE (table),
+				editor_new_placeholder (),
+				col, col + 1, row, row + 1,
+				GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL,
+				0, 0);
+	    }
+	}
+    }
+
+  /* Now make sure that if a row/col is occupied, any placeholders in that
+     row/col do not expand, and vice versa. */
+  children = GTK_TABLE (table)->children;
+  while (children)
+    {
+      child = children->data;
+      if (GB_IS_PLACEHOLDER (child->widget))
+	{
+	  gboolean xexpand = TRUE, yexpand = TRUE;
+	  gint row, col;
+
+	  /* Check if any widgets are in the same rows as the placeholder. */
+	  for (row = child->top_attach; row < child->bottom_attach; row++)
+	    {
+	      if (rows_occupied[row])
+		yexpand = FALSE;
+	    }
+	  child->yexpand = yexpand;
+
+	  /* Check if any widgets are in the same cols as the placeholder. */
+	  for (col = child->left_attach; col < child->right_attach; col++)
+	    {
+	      if (cols_occupied[col])
+		xexpand = FALSE;
+	    }
+	  child->xexpand = xexpand;
+
+	  child->xfill = TRUE;
+	  child->yfill = TRUE;
+	}
+      children = children->next;
+    }
+
+  g_free (rows_occupied);
+  g_free (cols_occupied);
+
+  gtk_widget_queue_resize (table);
+}
+
+
+/* Finds out if cell is occupied by a real widget. If not, returns whether
+   the cell is empty or whether a placeholder is in it */
+static gint
+gb_table_is_cell_occupied (GtkWidget * table, gint row, gint col)
+{
+  GList *children;
+  GtkTableChild *child;
+  gint return_val = GB_CELL_EMPTY;
+
+  children = GTK_TABLE (table)->children;
+  while (children)
+    {
+      child = children->data;
+      if (child->top_attach <= row && child->bottom_attach > row
+	  && child->left_attach <= col && child->right_attach > col)
+	{
+	  if (GB_IS_PLACEHOLDER (child->widget))
+	    return_val = GB_CELL_PLACEHOLDER;
+	  else
+	    return GB_CELL_WIDGET;
+	}
+      children = children->next;
+    }
+  return return_val;
+}
+
+
+static void
+gb_table_remove_placeholders (GtkWidget * table, gint row, gint col)
+{
+  GList *children, *next;
+  GtkTableChild *child;
+  gint left, right, top, bottom;
+
+  children = GTK_TABLE (table)->children;
+  while (children)
+    {
+      next = children->next;
+      child = children->data;
+      left = child->left_attach;
+      right = child->right_attach;
+      top = child->top_attach;
+      bottom = child->bottom_attach;
+
+      if (top <= row && bottom > row && left <= col && right > col)
+	{
+	  /* If the widget is a placeholder remove it */
+	  if (GB_IS_PLACEHOLDER (child->widget))
+	    {
+	      if (bottom - top > 1 || right - left > 1)
+		{
+		  gb_table_split_placeholder (table, child->widget,
+					      left, right, top, bottom,
+					      row, col);
+		}
+	      else
+		{
+		  gtk_container_remove (GTK_CONTAINER (table), child->widget);
+		}
+	    }
+	}
+      children = next;
+    }
+}
+
+
+static void
+gb_table_split_placeholder (GtkWidget * table, GtkWidget * placeholder,
+			    gint left, gint right, gint top, gint bottom,
+			    gint skip_row, gint skip_col)
+{
+  gint row, col;
+
+  gtk_container_remove (GTK_CONTAINER (table), placeholder);
+  for (row = top; row < bottom; row++)
+    {
+      for (col = left; col < right; col++)
+	{
+	  if (!(row == skip_row && col == skip_col))
+	    {
+	      gtk_table_attach (GTK_TABLE (table),
+				editor_new_placeholder (),
+				col, col + 1, row, row + 1,
+				GTK_FILL, GTK_FILL, 0, 0);
+	    }
+	}
+    }
+}
+
+
 /*
  * Writes the source code needed to create this widget.
  * You have to output everything necessary to create the widget here, though
@@ -655,6 +1040,63 @@ gb_table_write_source (GtkWidget * widget, GbWidgetWriteSourceData * data)
 }
 
 
+/* Outputs source to add a child widget to a table. */
+static void
+gb_table_write_add_child_source (GtkWidget * parent,
+				 const gchar *parent_name,
+				 GtkWidget *child,
+				 GbWidgetWriteSourceData * data)
+{
+  gchar xoptions[48], yoptions[48];
+  GtkTableChild *tchild = glade_util_find_table_child (GTK_TABLE (parent),
+						       child);
+  g_return_if_fail (tchild != NULL);
+
+  xoptions[0] = yoptions[0] = '\0';
+  if (tchild->xexpand)
+    strcpy (xoptions, "GTK_EXPAND");
+  if (tchild->xshrink)
+    {
+      if (xoptions[0] != '\0')
+	strcat (xoptions, " | ");
+      strcat (xoptions, "GTK_SHRINK");
+    }
+  if (tchild->xfill)
+    {
+      if (xoptions[0] != '\0')
+	strcat (xoptions, " | ");
+      strcat (xoptions, "GTK_FILL");
+    }
+  if (xoptions[0] == '\0')
+    strcpy (xoptions, "0");
+
+  if (tchild->yexpand)
+    strcpy (yoptions, "GTK_EXPAND");
+  if (tchild->yshrink)
+    {
+      if (yoptions[0] != '\0')
+	strcat (yoptions, " | ");
+      strcat (yoptions, "GTK_SHRINK");
+    }
+  if (tchild->yfill)
+    {
+      if (yoptions[0] != '\0')
+	strcat (yoptions, " | ");
+      strcat (yoptions, "GTK_FILL");
+    }
+  if (yoptions[0] == '\0')
+    strcpy (yoptions, "0");
+
+  source_add (data,
+	      "  gtk_table_attach (GTK_TABLE (%s), %s, %i, %i, %i, %i,\n"
+	      "                    (GtkAttachOptions) (%s),\n"
+	      "                    (GtkAttachOptions) (%s), %i, %i);\n",
+	      parent_name, data->wname,
+	      tchild->left_attach, tchild->right_attach,
+	      tchild->top_attach, tchild->bottom_attach,
+	      xoptions, yoptions, tchild->xpadding, tchild->ypadding);
+}
+
 
 /*
  * Initializes the GbWidget structure.
@@ -679,7 +1121,11 @@ gb_table_init ()
   gbwidget.gb_widget_create_properties = gb_table_create_properties;
   gbwidget.gb_widget_get_properties = gb_table_get_properties;
   gbwidget.gb_widget_set_properties = gb_table_set_properties;
+  gbwidget.gb_widget_create_child_properties = gb_table_create_child_properties;
+  gbwidget.gb_widget_get_child_properties = gb_table_get_child_properties;
+  gbwidget.gb_widget_set_child_properties = gb_table_set_child_properties;
   gbwidget.gb_widget_write_source = gb_table_write_source;
+  gbwidget.gb_widget_write_add_child_source = gb_table_write_add_child_source;
   gbwidget.gb_widget_create_popup_menu = gb_table_create_popup_menu;
 
   return &gbwidget;
