@@ -278,7 +278,7 @@ glade_gnome_get_stock_menu_pixmap_index (const gchar *icon_name)
  *************************************************************************/
 
 /* These were generated from gnome-stock.h with a Perl script, using
-   gnome-libs 1.0.1. */
+   gnome-libs 1.0.16. */
 const gchar *GladeStockMenuPixmapChoices[] =
 {
   N_("Blank"),
@@ -324,6 +324,7 @@ const gchar *GladeStockMenuPixmapChoices[] =
   N_("Line In"),
   N_("Cdrom"),
   N_("Volume"),
+  N_("Midi"),
   N_("Book Red"),
   N_("Book Green"),
   N_("Book Blue"),
@@ -396,6 +397,7 @@ const gchar *GladeStockMenuPixmapValues[] =
   GNOME_STOCK_MENU_LINE_IN,
   GNOME_STOCK_MENU_CDROM,
   GNOME_STOCK_MENU_VOLUME,
+  GNOME_STOCK_MENU_MIDI,
   GNOME_STOCK_MENU_BOOK_RED,
   GNOME_STOCK_MENU_BOOK_GREEN,
   GNOME_STOCK_MENU_BOOK_BLUE,
@@ -468,6 +470,7 @@ const gchar *GladeStockMenuPixmapSymbols[] =
   "GNOME_STOCK_MENU_LINE_IN",
   "GNOME_STOCK_MENU_CDROM",
   "GNOME_STOCK_MENU_VOLUME",
+  "GNOME_STOCK_MENU_MIDI",
   "GNOME_STOCK_MENU_BOOK_RED",
   "GNOME_STOCK_MENU_BOOK_GREEN",
   "GNOME_STOCK_MENU_BOOK_BLUE",
@@ -538,8 +541,7 @@ glade_gnome_get_stock_pixmap_index (const gchar *icon_name)
  *************************************************************************/
 
 /* These were generated from gnome-stock.h with a Perl script, using
-   gnome-libs 1.0.1. The "Midi" pixmap was added later. We may want to
-   regenerate these, though we'd lose backwards compatability.
+   gnome-libs 1.0.16.
    Note that ToolBarIndices in gnome/gnomeapp.c uses indexes into this array
    so if this is changed make sure they are updated. */
 const gchar *GladeStockPixmapChoices[] =
@@ -587,6 +589,7 @@ const gchar *GladeStockPixmapChoices[] =
   N_("Line In"),
   N_("Cdrom"),
   N_("Volume"),
+  N_("Midi"),
   N_("Book Red"),
   N_("Book Green"),
   N_("Book Blue"),
@@ -672,6 +675,7 @@ const gchar *GladeStockPixmapValues[] =
   GNOME_STOCK_PIXMAP_LINE_IN,
   GNOME_STOCK_PIXMAP_CDROM,
   GNOME_STOCK_PIXMAP_VOLUME,
+  GNOME_STOCK_PIXMAP_MIDI,
   GNOME_STOCK_PIXMAP_BOOK_RED,
   GNOME_STOCK_PIXMAP_BOOK_GREEN,
   GNOME_STOCK_PIXMAP_BOOK_BLUE,
@@ -757,6 +761,7 @@ const gchar *GladeStockPixmapSymbols[] =
   "GNOME_STOCK_PIXMAP_LINE_IN",
   "GNOME_STOCK_PIXMAP_CDROM",
   "GNOME_STOCK_PIXMAP_VOLUME",
+  "GNOME_STOCK_PIXMAP_MIDI",
   "GNOME_STOCK_PIXMAP_BOOK_RED",
   "GNOME_STOCK_PIXMAP_BOOK_GREEN",
   "GNOME_STOCK_PIXMAP_BOOK_BLUE",
@@ -1002,7 +1007,7 @@ glade_gnome_write_menu_item_source (GtkMenuItem * widget,
 	      gchar *label, *tooltip;
 
 	      label = glade_util_get_label_text (GTK_BIN (widget)->child);
-	      tooltip = data->widget_data->tooltip ? g_strdup (source_make_string (data->widget_data->tooltip, data->use_gettext)) : NULL;
+	      tooltip = data->widget_data->tooltip ? g_strdup (source_make_static_string (data->widget_data->tooltip, data->use_gettext)) : NULL;
 
 	      g_string_sprintfa (source_buffer,
 				 "  GNOMEUIINFO_MENU_NEW_ITEM (%s, %s, %s, NULL),\n",
@@ -1049,7 +1054,7 @@ glade_gnome_write_menu_item_source (GtkMenuItem * widget,
       gboolean free_pixmap_info;
       
       label = glade_util_get_label_text (GTK_BIN (widget)->child);
-      tooltip = data->widget_data->tooltip ? g_strdup (source_make_string (data->widget_data->tooltip, data->use_gettext)) : NULL;
+      tooltip = data->widget_data->tooltip ? g_strdup (source_make_static_string (data->widget_data->tooltip, data->use_gettext)) : NULL;
 
       /* Determine if the item has an icon and if it is a stock icon or an
 	 xpm file. */
