@@ -778,9 +778,11 @@ load_parse_color (GbWidgetSetArgData * data,
       return NULL;
     }
 
-  color.red = red << 8;
-  color.green = green << 8;
-  color.blue = blue << 8;
+  /* This effectively multiplies my 257 so we cover the range properly.
+     e.g. 00 -> 0000, 01 -> 0101, ff -> ffff. */
+  color.red = (red << 8) + red;
+  color.green = (green << 8) + green;
+  color.blue = (blue << 8) + blue;
   return &color;
 }
 
