@@ -18,8 +18,8 @@
  */
 
 #include <string.h>
-#include <sys/stat.h>
 #include <sys/types.h>
+#include <sys/stat.h>
 #include <stdarg.h>
 #include <unistd.h>
 #include <errno.h>
@@ -132,6 +132,7 @@ static void source_free_hash_keys_callback (gchar * key,
 					    gpointer data);
 static gchar* source_get_source_subdirectory (GbWidgetWriteSourceData * data);
 static void source_write_no_editing_warning (FILE *fp);
+static void source_write_include_files (FILE *fp);
 
 
 /* We need this so that numbers are written in C syntax rather than the
@@ -478,7 +479,7 @@ source_write_component (GtkWidget * component,
 
   fprintf (interface_c_fp,
 	   "GtkWidget*\n"
-	   "create_%s ()\n"
+	   "create_%s (void)\n"
 	   "{\n",
 	   data->component_name);
 
@@ -592,6 +593,20 @@ source_write_interface_h_preamble (GbWidgetWriteSourceData * data, FILE *fp)
   source_write_preamble (data->project_name, fp);
 }
 
+static void
+source_write_include_files (FILE *fp)
+{
+    fprintf (fp,
+	   "#ifdef HAVE_CONFIG_H\n"
+	   "#  include <config.h>\n"
+	   "#endif\n"
+	   "\n"
+	   "#include <sys/types.h>\n"
+	   "#include <sys/stat.h>\n"
+	   "#include <unistd.h>\n"
+	   "#include <string.h>\n"
+	   "\n");
+}
 
 /* Outputs the license and any include files needed at the top of the
    interface.c source file, before code for the components is output. */
@@ -601,15 +616,7 @@ source_write_interface_c_preamble (GbWidgetWriteSourceData * data, FILE *fp)
   source_write_no_editing_warning (fp);
   source_write_preamble (data->project_name, fp);
 
-  fprintf (fp,
-	   "#ifdef HAVE_CONFIG_H\n"
-	   "#  include <config.h>\n"
-	   "#endif\n"
-	   "\n"
-	   "#include <sys/stat.h>\n"
-	   "#include <unistd.h>\n"
-	   "#include <string.h>\n"
-	   "\n");
+  source_write_include_files (fp);
 
   if (glade_project_get_gnome_support (data->project))
     {
@@ -793,8 +800,10 @@ source_write_main_c (GbWidgetWriteSourceData * data)
   if (glade_project_get_gettext_support (data->project))
     {
       fprintf (fp,
+	       "#ifdef ENABLE_NLS\n"
 	       "  bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);\n"
 	       "  textdomain (PACKAGE);\n"
+	       "#endif\n"
 	       "\n");
     }
 
@@ -1420,8 +1429,8 @@ source_write_gnome_makefile_am (GbWidgetWriteSourceData * data)
   /* FIXME: If Makefile.am exists, just leave it, for now. */
   if (glade_util_file_exists (filename))
     {
-      return;
       g_free (filename);
+      return;
     }
 
   fp = fopen (filename, "w");
@@ -1949,7 +1958,12 @@ source_write_support_files (GbWidgetWriteSourceData * data)
     fprintf (fp, "#include <gnome.h>\n\n");
   else
     {
-      fprintf (fp, "#include <gtk/gtk.h>\n\n");
+      fprintf (fp,
+	       "#ifdef HAVE_CONFIG_H\n"
+	       "#  include <config.h>\n"
+	       "#endif\n"
+	       "\n"
+	       "#include <gtk/gtk.h>\n\n");
 
       /* For GTK+ apps that want gettext support, we define the standard
 	 macros here. */
@@ -2060,17 +2074,8 @@ source_write_support_files (GbWidgetWriteSourceData * data)
 
   source_write_no_editing_warning (fp);
   source_write_preamble (data->project_name, fp);
-
-  fprintf (fp,
-	   "#ifdef HAVE_CONFIG_H\n"
-	   "#  include <config.h>\n"
-	   "#endif\n"
-	   "\n"
-	   "#include <sys/stat.h>\n"
-	   "#include <unistd.h>\n"
-	   "#include <string.h>\n"
-	   "\n");
-
+  source_write_include_files (fp);
+ 
   if (glade_project_get_gnome_support (data->project))
     fprintf (fp, "#include <gnome.h>\n\n");
   else
