@@ -288,6 +288,7 @@ on_entry_dialog_ok (GtkWidget * widget, gpointer data)
   gchar *text;
   gboolean close = TRUE;
 
+  /* FIXME: ANSI forbids casting function pointer to data pointer. */
   handler = (GbEntryDialogFunc) (gtk_object_get_data (GTK_OBJECT (dialog),
 						      "handler"));
   widget = GTK_WIDGET (gtk_object_get_data (GTK_OBJECT (dialog), "widget"));
@@ -875,7 +876,7 @@ glade_util_find_layout_accelerator_target (GtkWidget *parent, GtkWidget *child)
   data.x = child->allocation.x;
   data.y = child->allocation.y;
   gtk_container_forall (GTK_CONTAINER (parent),
-			(gpointer)glade_util_find_layout_acclererator_target_cb,
+			(GtkCallback)glade_util_find_layout_acclererator_target_cb,
 			&data);
   return glade_util_find_focus_child (data.best_target);
 }
@@ -1181,13 +1182,14 @@ glade_util_copy_file (const gchar *src,
   input_fp = fopen (src, "r");
   if (input_fp == NULL)
     {
-      return glade_error_new_system ("Couldn't open file:\n  %s\n", src);
+      return glade_error_new_system (_("Couldn't open file:\n  %s\n"), src);
     }
 
   output_fp = fopen (dest, "w");
   if (output_fp == NULL)
     {
-      error = glade_error_new_system ("Couldn't create file:\n  %s\n", dest);
+      error = glade_error_new_system (_("Couldn't create file:\n  %s\n"),
+				      dest);
       fclose (input_fp);
       return error;
     }
@@ -1197,7 +1199,7 @@ glade_util_copy_file (const gchar *src,
       bytes_read = fread (buffer, 1, READ_BUFFER_SIZE, input_fp);
       if (bytes_read != READ_BUFFER_SIZE && ferror (input_fp))
 	{
-	  error = glade_error_new_system ("Error reading from file:\n  %s\n",
+	  error = glade_error_new_system (_("Error reading from file:\n  %s\n"),
 					  src);
 	  break;
 	}
@@ -1207,7 +1209,7 @@ glade_util_copy_file (const gchar *src,
 	  bytes_written = fwrite (buffer, 1, bytes_read, output_fp);
 	  if (bytes_read != bytes_written)
 	    {
-	      error = glade_error_new_system ("Error writing to file:\n  %s\n",
+	      error = glade_error_new_system (_("Error writing to file:\n  %s\n"),
 					      dest);
 	      break;
 	    }
@@ -1572,7 +1574,7 @@ glade_util_dirname (const gchar	   *file_name)
 
 /* This returns the parent directory of the given directory, which may or may
    not end in a G_DIR_SEPARATOR. If there is no parent directory, NULL is
-   returned. The returned string should be freed. */
+   returned. The returned string should be freed. dir MUST be absolute. */
 gchar*
 glade_util_parent_directory	(const gchar	    *dir)
 {
@@ -1630,7 +1632,7 @@ glade_util_get_next_free_project_directory (gchar **project_directory_return,
     {
       if (errno != ENOENT)
 	{
-	  error = glade_error_new_system ("Couldn't open directory:\n  %s\n",
+	  error = glade_error_new_system (_("Couldn't open directory:\n  %s\n"),
 					  projects_dir);
 	  g_free (projects_dir);
 	  return error;
